import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.*;
import javax.swing.filechooser.*;

public class AcceptedFileFilters extends FileFilter {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

    private static String TYPE_UNKNOWN = "Type Unknown";

    private Hashtable filters = null;
    private String description = null;
    private String fullDescription = null;
    private boolean useExtensionsInDescription = true;

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	/* Default constructor that accepts all file filters. */
    public AcceptedFileFilters() {
		this.filters = new Hashtable();
    }

	/* Constructor that accepts files with a given extension. */
    public AcceptedFileFilters( String extension ) {
		this( extension, null );
    }

    /**
     * Creates a file filter that accepts the given file type.
     * Example: new ExampleFileFilter("jpg", "JPEG Image Images");
     */
    public AcceptedFileFilters( String extension, String description ) {
		this();
		if ( extension != null ) addExtension( extension );
 		if ( description != null ) setDescription( description );
    }

/****************************************************************/
/*	Methods														*/
/****************************************************************/

    /**
     * Return true if this file should be shown in the directory pane,
     * false if it shouldn't.
     */
    public boolean accept( File f ) {
		if ( f != null ) {

	    	if ( f.isDirectory() ) {
				return true;
	    	}

	    	String extension = getExtension( f );
	    	if ( extension != null && filters.get( getExtension( f ) ) != null ) {
				return true;
	    	};
		}
		return false;
    }

/****************************************************************/

    /**
     * Return the extension portion of the file's name .
     */
    public String getExtension(File f) {
		if ( f != null ) {
	    	String filename = f.getName();
	    	int i = filename.lastIndexOf('.');

	    	if ( i>0 && ( i < filename.length() - 1 ) ) {
				return filename.substring(i+1).toLowerCase();
		    };
		}
		return null;
    }

/****************************************************************/

    /**
     * Adds a filetype "dot" extension to filter against.
     *
     * For example: the following code will create a filter that filters
     * out all files except those that end in ".jpg" and ".tif":
     *
     *   ExampleFileFilter filter = new ExampleFileFilter();
     *   filter.addExtension("jpg");
     *   filter.addExtension("tif");
     *
     * Note that the "." before the extension is not needed and will be ignored.
     */
    public void addExtension( String extension ) {
		if ( filters == null ) {
		    filters = new Hashtable( 5 );
		}
		filters.put( extension.toLowerCase(), this );
		fullDescription = null;
    }

/****************************************************************/

    /**
     * Returns the human readable description of this filter. For
     * example: "JPEG and GIF Image Files (*.jpg, *.gif)"
     */
    public String getDescription() {

		if ( fullDescription == null ) {
	    	if ( description == null || isExtensionListInDescription() ) {
 				fullDescription = description == null ? "(" : description + " (";

				// build the description from the extension list
				Enumeration extensions = filters.keys();

				if ( extensions != null ) {
				    fullDescription += "." + ( String ) extensions.nextElement();

		    		while ( extensions.hasMoreElements() ) {
						fullDescription += ", " + ( String ) extensions.nextElement();
		    		}
				}
				fullDescription += ")";
	    	}
	    	else {
			fullDescription = description;
	    	}
		}
		return fullDescription;
    }

/****************************************************************/

    /**
     * Sets the human readable description of this filter. For
     * example: filter.setDescription("Gif and JPG Images");
     */
    public void setDescription( String description ) {
		this.description = description;
		fullDescription = null;
    }

/****************************************************************/

    /**
     * Determines whether the extension list (.jpg, .gif, etc) should
     * show up in the human readable description.
     *
     * Only relevent if a description was provided in the constructor
     * or using setDescription();
     */
    public void setExtensionListInDescription( boolean b ) {
		useExtensionsInDescription = b;
		fullDescription = null;
    }

/****************************************************************/

    /**
     * Returns whether the extension list (.jpg, .gif, etc) should
     * show up in the human readable description.
     *
     * Only relevent if a description was provided in the constructor
     * or using setDescription();
     */
    public boolean isExtensionListInDescription() {
		return useExtensionsInDescription;
    }
}
